<?php

namespace Laravelista\Algo\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Storage;
use Illuminate\Validation\Rule;
use Laravelista\Picasso\Picasso;
use Laravelista\Algo\Helpers\Slug;

class UserController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $users = Config::get('algo.user')::orderBy('name')->get();

        return view('algo::users.index')->withUsers($users);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('algo::users.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request, Picasso $picasso, Slug $slug)
    {
        $validatedData = $request->validate([
            'name' => ['required', 'string', 'max:255'],
            'email' => ['required', 'string', 'email', 'max:255', 'unique:users'],
            'password' => ['required', 'string', 'min:8', 'confirmed'],
            'image' => ['nullable', 'file', 'mimes:jpeg,jpg,png'],
            'slug' => ['nullable', 'string', 'max:255', 'unique'],
            'location' => ['nullable', 'string', 'max:255'],
            'website' => ['nullable', 'string', 'max:255', 'url'],
            'facebook_profile' => ['nullable', 'string', 'max:255', 'url'],
            'twitter_profile' => ['nullable', 'string', 'max:255', 'url'],
            'bio' => ['nullable', 'string'],
        ]);

        $user = Config::get('algo.user')::create([
            'name' => $validatedData['name'],
            'email' => $validatedData['email'],
            'password' => Hash::make($validatedData['password'])
        ]);
        $user->slug = $slug->get($validatedData['slug'], $validatedData['name']);
        $user->location = $validatedData['location'];
        $user->website = $validatedData['website'];
        $user->facebook_profile = $validatedData['facebook_profile'];
        $user->twitter_profile = $validatedData['twitter_profile'];
        $user->bio = $validatedData['bio'];

        if ($request->hasFile('image')) {
            $user->image = $request->image->store('images', 'originals');
        }

        $user->save();

        if ($request->hasFile('image')) {
            $picasso->optimize($user->image, config('algo.user_dimensions'), 'public');
        }

        return redirect()->route('algo.users.index')->with('status', 'Korisnik napravljen!');
    }

    /**
     * Display the specified resource.
     *
     * @param  $user
     * @return \Illuminate\Http\Response
     */
    public function show($user)
    {
        $user = Config::get('algo.user')::findOrFail($user);

        return view('algo::users.show')->withUser($user);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  $user
     * @return \Illuminate\Http\Response
     */
    public function edit($user)
    {
        $user = Config::get('algo.user')::findOrFail($user);

        return view('algo::users.edit')->withUser($user);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  $user
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $user, Picasso $picasso, Storage $storage, Slug $slug)
    {
        $user = Config::get('algo.user')::findOrFail($user);

        $validatedData = $request->validate([
            'name' => ['required', 'string', 'max:255'],
            'email' => [
                'required', 'string', 'email', 'max:255',
                Rule::unique(Config::get('algo.user'))->ignore($user->id)
            ],
            'password' => ['nullable', 'string', 'min:8', 'confirmed'],
            'image' => ['nullable', 'file', 'mimes:jpeg,jpg,png'],
            'slug' => [
                'nullable', 'string', 'max:255',
                Rule::unique(Config::get('algo.user'))->ignore($user->id)
            ],
            'location' => ['nullable', 'string', 'max:255'],
            'website' => ['nullable', 'string', 'max:255', 'url'],
            'facebook_profile' => ['nullable', 'string', 'max:255', 'url'],
            'twitter_profile' => ['nullable', 'string', 'max:255', 'url'],
            'bio' => ['nullable', 'string'],
        ]);

        $user->fill([
            'name' => $validatedData['name'],
            'email' => $validatedData['email'],
        ]);
        $user->slug = $slug->get($validatedData['slug'], $validatedData['name']);
        $user->location = $validatedData['location'];
        $user->website = $validatedData['website'];
        $user->facebook_profile = $validatedData['facebook_profile'];
        $user->twitter_profile = $validatedData['twitter_profile'];
        $user->bio = $validatedData['bio'];

        if (!is_null($validatedData['password'])) {
            $user->password = Hash::make($validatedData['password']);
        }

        if ($request->hasFile('image') and $user->hasImage()) {
            $storage::disk('originals')->delete($user->image);
            $picasso->purge($user->image, 'public');
        }

        if ($request->hasFile('image')) {
            $user->image = $request->image->store('images', 'originals');
        }

        $user->save();

        if ($request->hasFile('image')) {
            $picasso->optimize($user->image, config('algo.user_dimensions'), 'public');
        }

        return redirect()
            ->route('algo.users.index')
            ->with('status', 'Korisnik uređen!');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  $user
     * @return \Illuminate\Http\Response
     */
    public function destroy($user, Storage $storage, Picasso $picasso)
    {
        $user = Config::get('algo.user')::findOrFail($user);

        if ($user->hasImage()) {
            $storage::disk('originals')->delete($user->image);
            $picasso->purge($user->image, 'public');
        }

        $user->delete();

        return redirect()
            ->route('algo.users.index')
            ->with('status', 'Korisnik obrisan!');
    }
}
